@@ -37,6 +37,7 @@ BRACKET_GLYPH_RE = re.compile(
         REVERSE_BRACKET_LABEL
     )
 )
+BRACKET_GLYPH_SUFFIX_RE = re.compile(r".*(\..*BRACKET\.\d+)$")
 
 
 class _LoggerMixin:
@@ -130,6 +131,13 @@ class UFOBuilder(_LoggerMixin):
         self._designspace = self.designspace_module.DesignSpaceDocument()
         self._designspace_is_complete = False
 
+        # If any glyph layer which is associated with a GSFontMaster has the
+        # properties "vertWidth" or "vertOrigin" set, Glyphsapp will assume
+        # the font is going to be used for vertical typesetting. When
+        # Glyphsapp generates these fonts, it will include the vhea, vmtx,
+        # VORG tables. VORG will only be included if the font is an otf.
+        self.is_vertical = self._is_vertical()
+
         # check that source was generated with at least stable version 2.3
         # https://github.com/googlefonts/glyphsLib/pull/65#issuecomment-237158140
         if int(font.appVersion) < 895:
@@ -160,6 +168,16 @@ class UFOBuilder(_LoggerMixin):
             # instances with matching 'familyName' custom parameter
             self._do_filter_instances_by_family = True
 
+    def _is_vertical(self):
+        master_ids = {m.id for m in self.font.masters}
+        for glyph in self.font.glyphs:
+            for layer in glyph.layers:
+                if layer.layerId not in master_ids:
+                    continue
+                if layer.vertWidth is not None or layer.vertOrigin is not None:
+                    return True
+        return False
+
     @property
     def masters(self):
         """Get an iterator over master UFOs that match the given family_name.
@@ -500,7 +518,6 @@ class UFOBuilder(_LoggerMixin):
     from .custom_params import to_ufo_custom_params
     from .features import to_ufo_features
     from .font import to_ufo_font_attributes
-    from .glyph import to_ufo_glyph, to_ufo_glyph_background
     from .groups import to_ufo_groups
     from .guidelines import to_ufo_guidelines
     from .hints import to_ufo_hints
@@ -511,6 +528,11 @@ class UFOBuilder(_LoggerMixin):
     from .names import to_ufo_names
     from .paths import to_ufo_paths
     from .sources import to_designspace_sources
+    from .glyph import (
+        to_ufo_glyph,
+        to_ufo_glyph_background,
+        to_ufo_glyph_height_and_vertical_origin,
+    )
     from .user_data import (
         to_designspace_family_user_data,
         to_ufo_family_user_data,
@@ -749,6 +771,19 @@ class GlyphsBuilder(_LoggerMixin):
                         )
                     )
                     setattr(source, name, getattr(source.font.info, name))
+        # An axis without a mapping will see its range information (min and max
+        # values) lost when converted to a Glyps.app file. To combat this we
+        # add an explicit identity mapping.
+        for axis in copy.axes:
+            if axis.map:
+                continue
+            if axis.minimum == axis.maximum:
+                axis.map = [(axis.minimum, axis.minimum)]
+            else:
+                axis.map = [
+                    (axis.minimum, axis.minimum),
+                    (axis.maximum, axis.maximum),
+                ]
         return copy
 
     def _fake_designspace(self, ufos):
@@ -825,7 +860,7 @@ class GlyphsBuilder(_LoggerMixin):
     from .custom_params import to_glyphs_custom_params
     from .features import to_glyphs_features
     from .font import to_glyphs_font_attributes, to_glyphs_ordered_masters
-    from .glyph import to_glyphs_glyph
+    from .glyph import to_glyphs_glyph, to_glyphs_glyph_height_and_vertical_origin
     from .groups import to_glyphs_groups
     from .guidelines import to_glyphs_guidelines
     from .hints import to_glyphs_hints
